<?xml version="1.0" encoding="iso-8859-1"?>
<!--
	 The contents of this file are subject to the terms of the
	 Common Development and Distribution License (the "License")
	 (see http://www.opensource.org/licenses/CDDL-1.0).

	 Copyright (c) 2011 Jens Elkner. All rights reserved.
	 Use is subject to license terms.
-->
<!--
	Enable SSL for admin port by modifying a default domain.xml for GF 3.1
	Usually one should use: "asadmin enable-secure-admin", however this 
	requires that the DAS is already running in un-secure mode...
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml"/>
<xsl:preserve-space  elements="*"/>

<xsl:template match="node()|@*">
<xsl:copy>
	<xsl:apply-templates select="node()|@*|text()|processing-instruction()|comment()"/>
</xsl:copy>
</xsl:template>

<!-- replace version attribute value with 43 -->
<xsl:template match="/domain">
	<domain>
		<xsl:copy-of select="@*"/>
		<xsl:attribute name="version">43</xsl:attribute>
		<xsl:apply-templates />
		<load-balancers/>
		<lb-configs/>
		<clusters/>
		<secure-admin enabled="true"/>
	</domain>
</xsl:template>

<!-- just for completness -->
<xsl:template match="/domain/configs/config">
	<config>
		<xsl:copy-of select="@*"/>
		<xsl:choose>
			<xsl:when test="./@name != 'default-config'">
				<xsl:apply-templates />
				<monitoring-service>
					<module-monitoring-levels/>
				</monitoring-service>
				<group-management-service>
					<failure-detection/>
				</group-management-service>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
				<monitoring-service>
					<module-monitoring-levels/>
				</monitoring-service>
				<connector-service/>
			</xsl:otherwise>
		</xsl:choose>
	</config>
</xsl:template>

<!-- add <das-config/> at the end -->
<xsl:template match="/domain/configs/config/admin-service">
	<admin-service>
		<xsl:copy-of select="@system-jmx-connector-name"/>
		<xsl:if test="../@name != 'default-config'">
			<xsl:copy-of select="@type"/>
		</xsl:if>
		<xsl:apply-templates />
		<das-config/>
	</admin-service>
</xsl:template>

<!-- strip off attributes: admin-password, admin-user-name, lazy-init -->
<xsl:template match="/domain/configs/config/jms-service/jms-host">
	<jms-host>
		<xsl:copy-of select="@name|@host|@port"/>
		<xsl:apply-templates />
	</jms-host>
</xsl:template>

<!-- replace URI scheme with 'https' -->
<xsl:template match="/domain/configs/config/security-service/message-security-config[@auth-layer='HttpServlet']/provider-config/property[@name='restAuthURL']">
	<property name="restAuthURL">
		<xsl:attribute name="value">
			<xsl:text>https:</xsl:text>
			<xsl:copy-of select="substring-after(@value,':')"/>
		</xsl:attribute>
	</property>
</xsl:template>

<!-- add additional protocols -->
<xsl:template match="/domain/configs/config/network-config/protocols">
	<protocols>
		<xsl:apply-templates />
		<protocol security-enabled="true" name="sec-admin-listener">
			<http default-virtual-server="__asadmin"
				encoded-slash-enabled="true">
				<file-cache/>
			</http>
			<ssl client-auth="want" ssl3-enabled="false" cert-nickname="s1as"
				classname="com.sun.enterprise.security.ssl.GlassfishSSLImpl">
			<xsl:if test="../../@name = 'default-config'">
				<xsl:attribute name="cert-nickname">glassfish-instance</xsl:attribute>
			</xsl:if>
			</ssl>
		</protocol>
		<protocol name="admin-http-redirect">
			<http-redirect secure="true"/>
		</protocol>
		<protocol name="pu-protocol">
			<port-unification>
				<protocol-finder protocol="sec-admin-listener"
					name="http-finder"
					classname="com.sun.grizzly.config.HttpProtocolFinder"/>
				<protocol-finder protocol="admin-http-redirect"
					name="admin-http-redirect"
					classname="com.sun.grizzly.config.HttpProtocolFinder"/>
			</port-unification>
		</protocol>
	</protocols>
</xsl:template>

<!-- set admin-listener protocol to pu-protocol -->
<xsl:template match="/domain/configs/config/network-config/network-listeners/network-listener[@name='admin-listener']">
	<network-listener>
		<xsl:copy-of select="@*"/>
		<xsl:attribute name="protocol">pu-protocol</xsl:attribute>
	</network-listener>
</xsl:template>

<!-- adjust JVM options to -server -d64 -->
<xsl:template match="/domain/configs/config/java-config/jvm-options">
	<xsl:choose>
		<xsl:when test="text() = '-client'">
			<jvm-options>-server</jvm-options>
		</xsl:when>
		<xsl:otherwise>
			<xsl:copy-of select="."/>
		</xsl:otherwise>
	</xsl:choose>	
</xsl:template>
<xsl:template match="/domain/configs/config/java-config">
	<java-config>
		<xsl:copy-of select="@*"/>
		<xsl:apply-templates />
		<jvm-options>-d64</jvm-options>
	</java-config>
</xsl:template>

</xsl:stylesheet>
